from typing import Dict, List, Optional

from pydantic import BaseModel

from lume.config.check_os_list_or_str_item import check_os_list_or_str_item


class InstallConfig(BaseModel):
    run: List[str]
    cwd: Optional[str] = None
    envs: Optional[Dict[str, str]] = None
    overwrote_envs: Optional[List[str]] = None

    def add_shared_env(self, shared_envs: Dict[str, str]):
        self.overwrote_envs = list(set(shared_envs.keys()).intersection(set(self.envs)))
        self.envs = {**shared_envs, **self.envs}

    @staticmethod
    def from_dict(kdict):
        run = check_os_list_or_str_item(kdict, "run")
        return InstallConfig(
            run=run if run else [], cwd=kdict.get("cwd"), envs=kdict.get("envs", {})
        )



from meiga import Error, Result, isSuccess

from lume.src.domain.services.executor_service import ExecutorService


class FakeExecutorService(ExecutorService):
    def execute(self, command: str, cwd: str) -> Result[bool, Error]:
        print("executing")
        return isSuccess



import os
from typing import Dict, List, Optional

import yaml
from pydantic import BaseModel
from yaml.parser import ParserError

from lume.config.check_list_or_str_item import check_list_or_str_item
from lume.config.check_os_list_or_str_item import check_os_list_or_str_item


def read_env_from_file(filename):
    if not filename or not os.path.isfile(filename):
        return {}
    try:
        with open(filename) as file:
            envs = yaml.load(file, Loader=yaml.FullLoader)
            return envs
    except ParserError:
        return {}


class StepConfig(BaseModel):
    run: List[str]
    cwd: Optional[str] = None
    envs: Optional[Dict[str, str]] = None
    setup: Optional[List[str]] = None
    teardown: Optional[List[str]] = None
    setup_detach: Optional[Dict] = None
    wait_seconds: Optional[int] = None
    wait_http_200: Optional[str] = None
    overwrote_envs: Optional[List[str]] = None

    def add_shared_env(self, shared_envs: Dict[str, str]):
        self.overwrote_envs = list(set(shared_envs.keys()).intersection(set(self.envs)))
        self.envs = {**shared_envs, **self.envs}

    @staticmethod
    def from_dict(kdict):
        run = check_os_list_or_str_item(kdict, "run", required=True)
        setup = check_os_list_or_str_item(kdict, "setup", required=False)
        teardown = check_os_list_or_str_item(kdict, "teardown", required=False)
        setup_detach = kdict.get("setup_detach")
        if setup_detach:
            setup_detach["run"] = check_list_or_str_item(
                setup_detach, "run", required=True, suffix=" (setup_detach)"
            )
        envs = kdict.get("envs", {})
        envs_from_file = read_env_from_file(kdict.get("envs_file"))
        envs.update(envs_from_file)

        return StepConfig(
            run=run,
            cwd=kdict.get("cwd"),
            envs=envs,
            setup=setup,
            teardown=teardown,
            setup_detach=setup_detach,
            wait_seconds=kdict.get("wait_seconds"),
            wait_http_200=kdict.get("wait_http_200"),
        )



from typing import Dict, List

import yaml

from lume.config.install_config import InstallConfig
from lume.config.setup_config import SetupConfig
from lume.config.step_config import StepConfig, read_env_from_file


class Config:
    def __init__(self, yaml_dict: Dict = None):
        if yaml_dict is None:
            self.name = None
            self.settings = {"show_exit_code": False}
            self.install = None
            self.steps = {}
        else:
            self.name = yaml_dict.get("name")
            self.settings = {
                "show_exit_code": yaml_dict.get("settings", {}).get(
                    "show_exit_code", False
                )
            }

            shared_envs = yaml_dict.get("envs", {})
            envs_from_file = read_env_from_file(yaml_dict.get("envs_file"))
            shared_envs.update(envs_from_file)

            if yaml_dict.get("install"):
                self.install = InstallConfig.from_dict(yaml_dict.get("install"))
            else:
                self.install = InstallConfig(run=[])
            self.install.add_shared_env(shared_envs)

            self.steps = {}
            for step_name, step in yaml_dict["steps"].items():
                if step_name == "setup":
                    self.steps[step_name] = SetupConfig(**step)
                else:
                    self.steps[step_name] = StepConfig.from_dict(step)
                    self.steps[step_name].add_shared_env(shared_envs)

            self.add_other_steps(yaml_dict, shared_envs)

    def get_steps(self) -> List[str]:
        return list(self.steps.keys())

    def get_commands(self) -> List[str]:
        commands = []
        commands += self.get_steps()
        if self.install:
            commands.append("install")
        return commands

    def add_other_steps(self, yaml_dict, shared_envs):
        other_steps = yaml_dict.get("other_steps", dict())
        for key, filename in other_steps.items():
            with open(filename) as file:
                yaml_dict = yaml.load(file, Loader=yaml.FullLoader)
                other_shared_envs = yaml_dict.get("envs", {})
                envs_from_file = read_env_from_file(yaml_dict.get("envs_file"))
                other_shared_envs.update(envs_from_file)
                shared_envs.update(other_shared_envs)
                for step_name, step in yaml_dict["steps"].items():
                    step_name = f"{key}:{step_name}"
                    self.steps[step_name] = StepConfig.from_dict(step)
                    self.steps[step_name].add_shared_env(shared_envs)
